@@ -5,29 +5,26 @@
 ## This program is free software. It may be copied and/or redistributed under
 ## the same terms as Perl itself.
 ##==============================================================================
-## $Id: Declaration.pm,v 0.11 2004/07/30 01:41:33 kevin Exp $
+## $Id: Declaration.pm,v 0.8 2004/07/17 04:34:18 kevin Exp $
 ##==============================================================================
 require 5.006;
 
-package Sub::Declaration;
+package ## don't index this yet
+	Sub::Declaration;
 use strict;
 use warnings;
-our ($VERSION) = q$Revision: 0.11 $ =~ /^Revision:\s+(\S+)/ or $VERSION = "0.0";
+our ($VERSION) = q$Revision: 0.8 $ =~ /^Revision:\s+(\S+)/ or $VERSION = "0.0";
 
-use Lexical::Util qw(frame_to_cvref lexical_alias ref_to_lexical);
+use Lexical::Util qw(frame_to_cvref lexalias ref_to_lexical);
 use Filter::Simple;
 use Symbol qw(qualify_to_ref);
-use attributes ();				## this seems to be needed for Perl2Exe
-use Carp;
-
-my ($Package, $File, $Line);
 
 ##==============================================================================
 ## Common regexen used below.
 ##==============================================================================
 my $id = qr/[_[:alpha:]]\w*/;
 my $item = qr/(?:(?:COPY\s+)?\\?[\$\@\%]|\\?[\&\*])$id/;
-my $arglist = qr/\s*$item(?:\s*[,;]\s*$item)*\s*(?:[,;]?\s*)?/;
+my $arglist = qr/$item\s*(?:[,;]\s*$item\s*)*\s*(?:[,;]?\s*)?/;
 
 ##==============================================================================
 ## Options used for each package
@@ -48,9 +45,8 @@ my @ArgumentLists;
 my %argspecs = (
 	'$'	     =>	[
 		'$', '$', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
-			my $msg = lexical_alias($cvref, $varname, \$_[0]);
-			confess "$filename($line): $msg" if $msg;
+			my ($cvref, $varname) = splice @_, -2;
+			lexalias($cvref, $varname, \$_[0]);
 			shift @_;
 			0;
 		},
@@ -58,7 +54,7 @@ my %argspecs = (
 	],
 	'COPY $' =>	[
 		'$', '$', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
+			my ($cvref, $varname) = splice @_, -2;
 			my $varref = ref_to_lexical($cvref, $varname);
 			$$varref = shift @_;
 			0;
@@ -66,16 +62,15 @@ my %argspecs = (
 	],
 	'\$'     =>	[
 		'$', '\$', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
-			my $msg = lexical_alias($cvref, $varname, \$$_[0]);
-			confess "$filename($line): $msg" if $msg;
+			my ($cvref, $varname) = splice @_, -2;
+			lexalias($cvref, $varname, \$$_[0]);
 			shift @_;
 			0;
 		},
 	],
 	'@'	     =>	[
 		'@', '@', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
+			my ($cvref, $varname) = splice @_, -2;
 			my $varref = ref_to_lexical($cvref, $varname);
 			@$varref = @_;
 			@_ = ();
@@ -84,7 +79,7 @@ my %argspecs = (
 	],
 	'COPY @' =>	[
 		'@', '\@', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
+			my ($cvref, $varname) = splice @_, -2;
 			my $varref = ref_to_lexical($cvref, $varname);
 			@$varref = @{shift @_};
 			0;
@@ -92,15 +87,14 @@ my %argspecs = (
 	],
 	'\@'     =>	[
 		'@', '\@', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
-			my $msg = lexical_alias($cvref, $varname, shift @_);
-			confess "$filename($line): $msg" if $msg;
+			my ($cvref, $varname) = splice @_, -2;
+			lexalias($cvref, $varname, shift @_);
 			0;
 		},
 	],
 	'%'	     =>	[
 		'%', '%', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
+			my ($cvref, $varname) = splice @_, -2;
 			my $varref = ref_to_lexical($cvref, $varname);
 			%$varref = @_;
 			@_ = ();
@@ -109,7 +103,7 @@ my %argspecs = (
 	],
 	'COPY %' =>	[
 		'%', '\%', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
+			my ($cvref, $varname) = splice @_, -2;
 			my $varref = ref_to_lexical($cvref, $varname);
 			%$varref = %{shift @_};
 			0;
@@ -117,15 +111,14 @@ my %argspecs = (
 	],
 	'\%'     =>	[
 		'%', '\%', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
-			my $msg = lexical_alias($cvref, $varname, shift @_);
-			confess "$filename($line): $msg\n" if $msg;
+			my ($cvref, $varname) = splice @_, -2;
+			lexalias($cvref, $varname, shift @_);
 			0;
 		},
 	],
 	'&'	     =>	[
 		'$', '&', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
+			my ($cvref, $varname) = splice @_, -2;
 			my $varref = ref_to_lexical($cvref, $varname);
 			$$varref = shift @_;
 			0;
@@ -133,7 +126,7 @@ my %argspecs = (
 	],
 	'\&'     =>	[
 		'$', '\&', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
+			my ($cvref, $varname) = splice @_, -2;
 			my $varref = ref_to_lexical($cvref, $varname);
 			$$varref = shift @_;
 			0;
@@ -141,7 +134,7 @@ my %argspecs = (
 	],
 	'*'	     =>	[
 		'$', '*', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
+			my ($cvref, $varname) = splice @_, -2;
 			my $varref = ref_to_lexical($cvref, $varname);
 			$$varref = qualify_to_ref(shift @_, (caller(2))[0]);
 			0;
@@ -149,7 +142,7 @@ my %argspecs = (
 	],
 	'\*'     =>	[
 		'$', '\*', sub {
-			my ($cvref, $varname, $filename, $line) = splice @_, -4;
+			my ($cvref, $varname) = splice @_, -2;
 			my $varref = ref_to_lexical($cvref, $varname);
 			$$varref = qualify_to_ref(shift @_, (caller(2))[0]);
 			0;
@@ -176,17 +169,17 @@ sub arg2var ($) {
 }
 
 ##==============================================================================
-## ensub($name, $arglist);
+## ensub($package, $name, $arglist);
 ## Convert a 'method' declaration to the equivalent 'sub' declaration, adding
 ## the object reference to the front of the argument list.
 ##==============================================================================
 sub ensub {
-	my ($name, $arglist) = @_;
+	my ($package, $name, $arglist) = @_;
 	my $output = 'method ';
 	$output .= $name if defined $name;
 
-	my $selfvar = $Options{$Package}{'self'} || 'self';
-	$selfvar = 'COPY $' . $selfvar;
+	my $selfvar = $Options{$package}{'self'} || 'self';
+	$selfvar = '$' . $selfvar;
 
 	$arglist = defined $arglist ? "$selfvar, " . $arglist : $selfvar;
 
@@ -194,19 +187,19 @@ sub ensub {
 }
 
 ##==============================================================================
-## protoize($type, $name, $arglist, $attrs, $punct);
+## protoize($package, $type, $name, $arglist, $attrs, $punct);
 ## after matching 'sub <name>(<arglist>) <punct>', output the appropriate
 ## conversion.
 ##==============================================================================
 sub protoize {
-	my ($type, $name, $arglist, $attrs, $punct) = @_;
+	my ($package, $type, $name, $arglist, $attrs, $punct) = @_;
 	my $output = 'sub ';
 	$output .= $name if defined $name;
 
 	my @args = split m/\s*[,;]\s*/, $arglist;
 	if ($type eq 'method') {
-		$output .= ' : method' if $Options{$Package}{'method'};
-	} elsif ($Options{$Package}{'proto'}) {
+		$output .= ' : method' if $Options{$package}{'method'};
+	} elsif ($Options{$package}{'proto'}) {
 		my $proto = join '', map { (arg2var($_))[1] } @args;
 		$output .= "($proto)";
 	}
@@ -225,12 +218,12 @@ sub protoize {
 }
 
 ##==============================================================================
-## expand_arglist($arglist);
+## expand_arglist($package, $arglist);
 ## Convert argument list into 'my' declaration followed by commands to set them
 ## up (alias, copy, or slurp).
 ##==============================================================================
 sub expand_arglist {
-	my ($arglist) = @_;
+	my ($package, $arglist) = @_;
 	my $output;
 	my @args = split m/\s*[,;]\s*/, $arglist;
 	my @vars = map {
@@ -257,15 +250,17 @@ sub expand_arglist {
 ## FILTER
 ##==============================================================================
 FILTER {
+	my $package = caller(1);
+
 	##--------------------------------------------------------------------------
 	## Convert any 'method' declarations into the equivalent 'sub' declarations.
 	##--------------------------------------------------------------------------
 	s{
-        (?<!\S)\bmethod\b               ## Must not immediately follow non-WS
+        (?<![\$\@\%\*\&\\\:])\bmethod\b ## But not if it's a variable
         (?:\s+($id(?:::$id)*))?\s*      ## Name into $1 if present
         (?:\(($arglist)\))?             ## Capture arg list into $2
 	}{
-		ensub($1, $2)
+		ensub($package, $1, $2)
 	}sexg;
 
 	##--------------------------------------------------------------------------
@@ -273,13 +268,14 @@ FILTER {
 	## 'argument list' directives.
 	##--------------------------------------------------------------------------
 	s{
-        (?<!\S)\b(sub|method)\b         ## "sub" or "method" into $1
+        (?<![\$\@\%\*\&\\\:])           ## But not if it's a variable
+        \b(sub|method)\b                ## Capture type into $1
         (?:\s+($id(?:::$id)*))?\s*      ## Name into $2 if present
         \(($arglist)\)\s*               ## Capture arg list into $3
         (?:(:\s*[^\x7B;]+)\s*)?			## Attribute list into $4
         ([\x7B;])                       ## Save terminating punctuation into $5
 	}{
-		protoize($1, $2, $3, $4, $5)
+		protoize($package, $1, $2, $3, $4, $5)
 	}sexg;
 
 	##--------------------------------------------------------------------------
@@ -288,10 +284,11 @@ FILTER {
 	## explicitly.
 	##--------------------------------------------------------------------------
 	s{
-        (?<!\S)\bargument\s+list\b\s*   ## Must not immediately follow non-WS
+        (?<![\$\@\%\*\&\\])             ## Not if it's a variable
+        \bargument\s+list\s*            ## Initial prefix
         \(($arglist)\)\s*               ## Argument list into $1
 	}{
-		expand_arglist($1)
+		expand_arglist($package, $1)
 	}sexg;
 
 	##--------------------------------------------------------------------------
@@ -299,7 +296,7 @@ FILTER {
 	##--------------------------------------------------------------------------
 	s{\\(sub|method|argument\s+list)\b}{$1}sg;
 
-	print if $Options{$Package}{'debug'};
+	print if $Options{$package}{'debug'};
 };
 
 ##==============================================================================
@@ -308,11 +305,10 @@ FILTER {
 sub arglist {
 	my $arg_index = pop @_;
 	my $cvref = frame_to_cvref(1);
-	my ($package, $filename, $line) = caller;
 
 	foreach (@{$ArgumentLists[$arg_index]}) {
 		my ($varname, $proc) = @$_;
-		push @_, $cvref, $varname, $filename, $line;
+		push @_, $cvref, $varname;
 		last if &$proc;
 	}
 }
@@ -322,25 +318,25 @@ sub arglist {
 ##==============================================================================
 sub import {
 	my $class = shift;
-	($Package, $File, $Line) = caller;
+	my ($package, $filename, $line) = caller;
 
-	$Options{$Package} = {
+	$Options{$package} = {
 		'self' => 'self',
 		'proto' => 1,
 		'debug' => 0,
 		'method' => 1,
-	} unless exists $Options{$Package};
+	} unless exists $Options{$package};
 
 	foreach (@_) {
 		/^-(no)?(proto|method|debug)$/ and do {
-			$Options{$Package}{$2} = !defined($1);
+			$Options{$package}{$2} = !defined($1);
 			next;
 		};
 		/^-self:($id)$/ and do {
-			$Options{$Package}{'self'} = $1;
+			$Options{$package}{'self'} = $1;
 			next;
 		};
-		die "$File($Line): invalid import option '$_'\n";
+		die "$filename($line): invalid import option '$_'\n";
 	}
 }
 
@@ -348,17 +344,6 @@ sub import {
 
 ##==============================================================================
 ## $Log: Declaration.pm,v $
-## Revision 0.11  2004/07/30 01:41:33  kevin
-## Use 'lexical_alias' routine for better error messages.
-## Improve the regular expressions used for filtering.
-##
-## Revision 0.10  2004/07/29 01:31:49  kevin
-## Fix option handling, the regexen that detect the special syntax, and go ahead
-## and let CPAN index this.
-##
-## Revision 0.9  2004/07/27 02:13:05  kevin
-## POD and README changes only.
-##
 ## Revision 0.8  2004/07/17 04:34:18  kevin
 ## Try to keep everything on one line.
 ##
@@ -387,8 +372,6 @@ sub import {
 
 __END__
 
-=pod
-
 =head1 NAME
 
 Sub::Declaration - declare subs with named parameters
@@ -789,20 +772,13 @@ B<arglist> subroutine within this module. This I<must> be called using the
 C<&>-form of subroutine call with no arguments and no trailing parentheses,
 after pushing a reference to an array containing the names of the variables in
 the argument list. There is almost certainly no reason whatsoever for you to
-have to call this routine explicitly, so it is not documented further.
+have to call this routine explicitly.
 
 =head1 MODULES USED
 
-L<Filter::Simple|Filter::Simple>, 
-L<Lexical::Util|Lexical::Util> (version 0.8 or later)
-
-=head1 SEE ALSO
-
-For other approaches, see:
-
-L<Perl6::Parameters|Perl6::Parameters>,
-L<Sub::Parameters|Sub::Parameters>,
-L<Sub::NamedParams|Sub::NamedParams>
+L<Filter::Simple|Filter::Simple>,
+L<Lexical::Util|Lexical::Util>,
+L<Symbol|Symbol>
 
 =head1 COPYRIGHT AND LICENSE
 
@@ -813,4 +789,4 @@ same terms as Perl itself.
 
 =head1 AUTHOR
 
-Kevin Michael Vail <F<kvail>@F<cpan>.F<org>>
+Kevin Michael Vail <F<kevin>@F<vaildc>.F<net>>
